#include <formatio.h>
#include "rsvna.h"
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "rsvna_one_and_two_port_calibration_example.h"


#define ARRAY_SIZE    5000
#define STRING_LENGTH 512
#define WINDOW        1

/* Useful macros */
#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else


/* Global variables */
static   ViInt32    panelHandle  = 0;
static   ViSession  instrSession = VI_NULL;
    

/* Utility functions declarations */
ViStatus checkError(ViStatus status);
void cleanup (ViBoolean deviceClear);
void EnableButtons(ViBoolean enable);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "rsvna_one_and_two_port_calibration_example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

int CVICALLBACK panelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_CLOSE:
            /* cleanup specan driver session */
            cleanup(VI_FALSE);
            
            QuitUserInterface (0);
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Start Button Callback
/* Purpose:  Configures the instrument and initiates the measurement
/*===========================================================================*/
int CVICALLBACK startButton (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar      instr_desc[STRING_LENGTH] = "";
    ViInt32     id               = 0;
    ViInt32     reset            = VI_FALSE;
    ViInt32     display          = VI_FALSE;
    ViStatus    status           = VI_SUCCESS;
    ViInt32     trace            = 0;
    ViInt32     buffer_size      = STRING_LENGTH;
    ViChar      cal_kit_name[STRING_LENGTH] = "";
    ViChar      save_calibration[STRING_LENGTH] = "";
    ViInt32     two_port_selected = 0;
    ViInt32     connector_type   = 0;
    ViInt32     port_one_conn    = 0;
    ViInt32     port_two_conn    = 0;
    
    switch (event)
    {
        case EVENT_COMMIT:     /* start button pressed */
        {
        
            /* block mouse events */
            SetWaitCursor (1);
            
            /* read values from panel*/
            GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
            GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
            GetCtrlVal (panelHandle, PANEL_RESET, &reset);
            GetCtrlVal (panelHandle, PANEL_DISPLAYON, &display);
            GetCtrlVal (panelHandle, PANEL_CAL_KIT_NAME, cal_kit_name);
            GetCtrlVal (panelHandle, PANEL_SAVE_CALIBRATION, save_calibration);
            GetCtrlVal (panelHandle, PANEL_PORT_SELECTOR, &two_port_selected);
            GetCtrlVal (panelHandle, PANEL_CONN_TYPE, &connector_type);
            GetCtrlVal (panelHandle, PANEL_PORT_ONE_CONN, &port_one_conn);
            GetCtrlVal (panelHandle, PANEL_PORT_TWO_CONN, &port_two_conn);
            
            /* disable buttons for configuration */
            EnableButtons(VI_FALSE);
            
            /* initialize driver */
            CHECKERR (rsvna_init (instr_desc, id, reset, &instrSession));
            
            /* switch device display on/off */
            CHECKERR (rsvna_SetAttributeViInt32 (instrSession, "", RSVNA_ATTR_DISPLAY_UPDATE, display));
            
            /* Set cal kit as active kit for N50 */
            CHECKERR (rsvna_ConfigureCalibrationKit (instrSession, connector_type, cal_kit_name));
            
            /* Select connectors for the ports*/
            CHECKERR (rsvna_ConfigureConnector (instrSession, 1, 1, port_one_conn));
            
            if (two_port_selected == 1)
            {
                /* Select connectors for the ports */
                CHECKERR (rsvna_ConfigureConnector (instrSession, 1, 2, port_two_conn));
            }
            
            /* Deactivates the calibration mode where the raw measurement data 
            of the standards is stored after the calibration is completed. */
            CHECKERR (rsvna_SetAttributeViBoolean (instrSession, "Channel1", RSVNA_ATTR_REPEAT_PREVIOUS_CALIBRATION, VI_FALSE));
            
            if (two_port_selected == 0)
            {
                /* Full one port = OSM
                Select cal procedure */
                CHECKERR (rsvna_SelectCalibrationType (instrSession, 1, "Test SFK OSM 1", RSVNA_VAL_CALTYPE_FOP, 1, 0, 0, 0));
                
                /* Measure Standards */
                MessagePopup ("One-Port Calibration", "Connect/disconnect calibration unit for OPEN calibration step.");
                CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_OPEN, 1, 0));
                MessagePopup ("One-Port Calibration", "Connect/disconnect calibration unit for SHORT calibration step.");
                CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_SHORT, 1, 0));
                MessagePopup ("One-Port Calibration", "Connect/disconnect calibration unit for MATCH calibration step.");
                CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_MATCH, 1, 0));
            
                /* Apply calibration */
                CHECKERR (rsvna_SaveCalibrationData (instrSession, 1));
            }
            
            if (two_port_selected == 1)
            {
                /* 2 port TOSM
                Select cal procedure */
                CHECKERR (rsvna_SelectCalibrationType (instrSession, 1, "Test SFK TOSM 12", RSVNA_VAL_CALTYPE_TOSM, 1, 2, 0, 0));
                
                /* Measure Standards */
                MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for THROUGH calibration step.");
                CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_THR, 1, 2));
                MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for OPEN port 1 calibration step.");
                CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_OPEN, 1, 0));
                MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for SHORT port 1 calibration step.");
                CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_SHORT, 1, 0));
                MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for MATCH port 1 calibration step.");
                CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_MATCH, 1, 0));
                MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for OPEN port 2 calibration step.");
                CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_OPEN, 2, 0));
                MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for SHORT port 2 calibration step.");
                CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_SHORT, 2, 0));
                MessagePopup ("Two-Port Calibration", "Connect/disconnect calibration unit for MATCH port 2 calibration step.");
                CHECKERR (rsvna_StartCalibration (instrSession, 1, RSVNA_VAL_CALCOLLSTDGEN_MATCH, 2, 0));
            
                /* Apply calibration */
                CHECKERR (rsvna_SaveCalibrationData (instrSession, 1)); 
            }
            /* Save cal file
            Save calibration in calibration file pool in directory
            C:\Rohde&Schwarz\Nwa\Calibration\Data
            the file name in the commands must not contain the path ! */
            CHECKERR (rsvna_CalibrationManager (instrSession, 1, RSVNA_VAL_CORR_MAN_COPY, save_calibration, ""));
                
            /* close session */
            cleanup(VI_FALSE);
        }
        break;
    }
    return 0;
}


/*===========================================================================*/
/* Function: checkError
/* Purpose:  If passed status indicates an failure, the error message will
/*           be displayed.
/*===========================================================================*/
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH*2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        /* Converts a status code returned by an instrument driver function into a user-readable string */
        rsvna_error_message (instrSession, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsvna_error_query (instrSession, &error, error_message);
        
        SetWaitCursor (0);
        if (error != VI_SUCCESS)
        {
            sprintf (p2buf, "Secondary Error: 0x%08X, %s\n", error, error_message);
        }
        MessagePopup ("Error", error_buffer);
        
        /* close session */
        cleanup(VI_TRUE);
        
        /* enable buttons for configuration */
        EnableButtons(VI_TRUE);
    }
    return status;
}


/*===========================================================================*/
/* Function: EnableButtons
/* Purpose:  This function dis-/enables GUI while measuring/configuration.
/*===========================================================================*/
void EnableButtons(ViBoolean enable)
{
    SetCtrlAttribute (panelHandle, PANEL_START,            ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_RESET,            ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_IDQUERY,          ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_INSTR_DESC,       ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_DISPLAYON,        ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_CAL_KIT_NAME,     ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_SAVE_CALIBRATION, ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_PORT_SELECTOR,    ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_CONN_TYPE,        ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_PORT_ONE_CONN,    ATTR_DIMMED, !enable);
    SetCtrlAttribute (panelHandle, PANEL_PORT_TWO_CONN,    ATTR_DIMMED, !enable);
}


/*===========================================================================*/
/* Function: Cleanup
/* Purpose:  Closes the VISA session to the instrument and in case of error
/*           queries the error and shows the dialog box with the passed error
/*           message. In case Device Clear is requested, it is necessary to 
/*           set the argument deviceClear to VI_TRUE.
/*===========================================================================*/
void cleanup (ViBoolean deviceClear)
{

    /* Device Clear is a low-level command and should be used in case the
     * instrument is waiting for Operation Complete to cancel the wait.
     * It is useful for instance in case of incorrect external trigger
     * when the instrument does not respond to any other command because of
     * waiting for trigger.
     */
    if (deviceClear)
    {
        viClear (instrSession);
        viPrintf (instrSession, "*CLS\n");
    }
    
    /* close instrument session */
    rsvna_close (instrSession);
    instrSession = VI_NULL;
    
    /* enable GUI */
    EnableButtons (1);
    /* enable mouse events*/
    SetWaitCursor (0);
}


